import React, { useEffect, useState } from 'react'
import { Helmet } from 'react-helmet'
import { Link, useLocation, useParams } from 'react-router-dom'
import axios from 'axios'
import { API_BASE_URL, SITE_TITLE } from '@/config'
import NicoViewer from '@/components/NicoViewer'
import { Button } from '@/components/ui/button'
import { toast } from '@/components/ui/use-toast'
import { cn } from '@/lib/utils'
import MainArea from '@/components/layout/MainArea'
import TagDetailSidebar from '@/components/TagDetailSidebar'
import PostEditForm from '@/components/PostEditForm'
import TabControl from '@/components/common/TabControl'

import type { Post, Tag, User } from '@/types'

type Props = { user: User }


export default ({ user }: Props) => {
  const { id } = useParams ()

  const location = useLocation ()

  const [post, setPost] = useState<Post | null> (null)
  const [editing, setEditing] = useState (true)

  const changeViewedFlg = () => {
    if (post?.viewed)
      {
        void (axios.delete (
                  `${ API_BASE_URL }/posts/${ id }/viewed`,
                  { headers: { 'X-Transfer-Code': localStorage.getItem ('user_code') || '' } })
              .then (res => setPost (post => {
                post.viewed = false
                return post
              }))
              .catch (err => toast ({ title:        '失敗……',
                                      description:  '通信に失敗しました……' })))
      }
    else
      {
        void (axios.post (
                  `${ API_BASE_URL }/posts/${ id }/viewed`,
                  { },
                  { headers: { 'X-Transfer-Code': localStorage.getItem ('user_code') || '' } })
              .then (res => setPost (post => {
                post.viewed = true
                return post
              }))
              .catch (err => toast ({ title:        '失敗……',
                                      description:  '通信に失敗しました……' })))
      }
  }

  useEffect (() => {
    if (!(id))
      return
    void (axios.get (`${ API_BASE_URL }/posts/${ id }`, { headers: {
              'X-Transfer-Code': localStorage.getItem ('user_code') || '' } })
          .then (res => setPost (res.data))
          .catch (err => console.error ('うんち!', err)))
  }, [id])

  useEffect (() => {
    setEditing (false)
  }, [post])

  useEffect (() => {
    if (!(editing))
      setEditing (true)
  }, [editing])

  const url = post ? new URL (post.url) : undefined

  return (
      <>
        <Helmet>
          {post && <title>{`${ post.title || post.url } | ${ SITE_TITLE }`}</title>}
        </Helmet>
        <TagDetailSidebar post={post} />
        <MainArea>
          {post
           ? (
              <div className="p-4">
                {(() => {
                  if (url.hostname.split ('.').slice (-2).join ('.') === 'nicovideo.jp')
                    {
                      return (
                          <NicoViewer
                            id={url.pathname.match (
                                  /(?<=\/watch\/)[a-zA-Z0-9]+?(?=\/|$)/)[0]}
                            width="640"
                            height="360" />)
                    }
                  else
                    return <img src={post.thumbnail} alt={post.url} className="mb-4 w-full" />
                }) ()}
                <Button onClick={changeViewedFlg}
                        className={cn ('text-white',
                                       post.viewed ? 'bg-blue-600 hover:bg-blue-700' : 'bg-gray-500 hover:bg-gray-600')}>
                  {post.viewed ? '閲覧済' : '未閲覧'}
                </Button>
                {(['admin', 'member'].includes (user.role) && editing) &&
                    <TabControl tabs={{
                        ['編輯']: (
                            <PostEditForm post={post}
                                          onSave={newPost => {
                                            setPost (newPost)
                                            toast ({ description: '更新しました．' })
                                          }} />) }} />}
              </div>)
           : 'Loading...'}
        </MainArea>
      </>)
}
